package de.bsvrz.sba.plugin.actions.parameter;

import java.util.LinkedList;

import org.eclipse.jface.action.Action;

/**
 * Dieses Interface definiert eine Action, welche Parameter vom Typ
 * {@link IParameter} verarbeiten kann. Der Entwickler hat sicherzustellen, dass
 * seine {@link IParameterAction} beim Austausch von {@link IParameter} mit den
 * gelieferten/zu liefernden {@link IParameter}n auch umgehen kann, Abstrze
 * sind zu vermeiden. Eine {@link IParameterAction} wird ber eine Plug-In ID ({@link IParameterAction#getPlugInId()})
 * und eine ParameterActionID ({@link IParameterAction#getParameterActionId()})
 * eindeutig identifiziert. <br>
 * Die Methide {@link IParameterAction#runParameterAction(LinkedList)} bernimmt
 * eine Liste von {@link IParameter}s und fhrt die hinterlege {@link Action}
 * aus. Die Ausfhrung dieser Methode ist Synchronized <br>
 * In der Error Log View erscheinen Warnhinweise, wenn eine
 * {@link IParameterAction} mit der gleichen ID vom gleichen Contributor erzeugt
 * wurde, auf. Dies muss der Plug-In Entwickler beobachten und bei Auftreten
 * eine andere ID vergeben!
 * 
 * @author HofmannJ
 * 
 */
public interface IParameterAction
{
	/**
	 * Liefert die ID dieser {@link IParameterAction} in Bezug auf ihr Plug-In.<br>
	 * Durch die Kombination von Plug-In ID und ParameterActionID wird diese
	 * {@link IParameterAction} eindeutig identifiziert.
	 * 
	 * @return Parameter Action ID
	 */
	public String getParameterActionId();

	/**
	 * Liefert eine Liste der {@link IParameter}-IDs, welche von der
	 * {@link IParameterAction} verarbeitet werden knnen.
	 * 
	 * @return Liste von Strings mit den {@link IParameter}-IDs
	 */
	public LinkedList<String> getParameterIds();
	
	/**
	 * Liefert eine Liste der aktuell gesetzten Parameter.
	 * @return Liste der aktuell gesetzten Parameter
	 */
	public LinkedList<IParameter> getParameters();
	
	/**
	 * Setzt die Liste der Parameter. Evtl. vorhandene Parameter mit der gleichen ID werden ersetzt.
	 * @param parameters Liste der zu setzenden Parameter.
	 */
	public void setParameters(LinkedList<IParameter> parameters);
	
	
	/**
	 * Entfernt alle aktuell gesetzten Parameter.
	 */
	public void clearParameters();
	
	
	/**
	 * Liefert die {@link Action} dieser {@link IParameterAction} zurck.
	 * @return {@link Action} dieser {@link IParameterAction}
	 */
	public Action getParameterAction();

	/**
	 * Fhrt die {@link IParameterAction} mit einer Liste von {@link IParameter}
	 * aus. In der Implementierung <b>muss</b> durch den Entwickler darauf
	 * geachtet werden, dass nur ein Thread zur gleichen Zeit diese Methode
	 * aufrufen kann.
	 * 
	 * @param parameters
	 *          Liste von {@link IParameter}
	 * @return Liefert <code>true</code>, wenn die {@link IParameterAction}
	 *         ausgefhrt werden knnen, sonst <code>false</code>
	 */
	public boolean runParameterAction(LinkedList<IParameter> parameters);
}
