/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.sba.plugin.navigator.navigatorView;


import org.eclipse.jface.viewers.ITreeContentProvider;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.swt.widgets.Display;
import org.eclipse.ui.PlatformUI;

import de.bsvrz.sba.plugin.navigator.navigatorModell.DeltaEvent;
import de.bsvrz.sba.plugin.navigator.navigatorModell.IDeltaListener;
import de.bsvrz.sba.plugin.navigator.navigatorModell.INavigatorElement;

/**
 * Bereitet die Inhalte des TreeViewers auf. {@link ITreeContentProvider}
 * 
 * @author Krause
 * 
 */
public class NavigatorContentProvider implements ITreeContentProvider, IDeltaListener
{
  /**
   * Flag zum Debuggen
   */
  protected static final boolean _debug = false;

  /**
   * Baumstruktur selbst 
   */
  private TreeViewer m_viewer;

  /**
   * ein NavigatorElement 
   */
  private Object m_navElement;

  /**
   * Liefert die Kinder eines Elements
   */
  public Object[] getChildren( Object parentElement )
  {
    if ( parentElement instanceof INavigatorElement )
      return ((INavigatorElement) parentElement).getChildren();
    return new Object[0];
  }

  /**
   * Liefert den Elternteil eines Elements
   */
  public Object getParent( Object element )
  {
    if ( element instanceof INavigatorElement )
      return ((INavigatorElement) element).getParent();
    return null;
  }

  /**
   * Liefert, ob ein Element Kinder hat
   */
  public boolean hasChildren( Object element )
  {
    // Kinder holen
    Object[] obj = getChildren( element );

    // Rckgabe, ob das Element Kinder hat
    return obj == null ? false : obj.length > 0;
  }

  /**
   * Liefert die Root-Elemente, kann daher meist getChildren() aufrufen
   */
  public Object[] getElements( Object inputElement )
  {
    return getChildren( inputElement );
  }

  /**
   * Verwerfen aller erzeugter Ressourcen
   */
  public void dispose()
  {
  }

  /**
   * Called when the input changes
   * 
   * @param arg0
   *            the viewer
   * @param arg1
   *            the old input
   * @param arg2
   *            the new input
   */
  public void inputChanged( Viewer viewer, Object oldInput, Object newInput )
  {
    if ( _debug )
      System.out.println( "InputChanged: " + newInput );

    this.m_viewer = (TreeViewer) viewer;
    if ( oldInput != null && newInput instanceof INavigatorElement )
      removeListenerFrom( (INavigatorElement) oldInput );
    if ( newInput != null && newInput instanceof INavigatorElement )
      addListenerTo( (INavigatorElement) newInput );
  }

  /** Because the domain model does not have a richer
   * listener model, recursively remove this listener
   * from each child. */
  protected void removeListenerFrom( INavigatorElement element )
  {
    element.removeListener( this );
    Object[] children = element.getChildren();
    for (int i = 0; i < children.length; i++)
    {
      INavigatorElement aElement = (INavigatorElement) children[i];
      removeListenerFrom( aElement );
    }
  }

  /** Because the domain model does not have a richer
   * listener model, recursively add this listener
   * to each child. */
  protected void addListenerTo( INavigatorElement element )
  {
    element.addListener( this );
    Object[] children = element.getChildren();
    for (int i = 0; i < children.length; i++)
    {
      INavigatorElement aElement = (INavigatorElement) children[i];
      addListenerTo( aElement );
    }
  }

  /*
   * @see IDeltaListener#add(DeltaEvent)
   */
  public void add( DeltaEvent event )
  {
    m_navElement = ((INavigatorElement) event.receiver()).getParent();
    if ( _debug )
      System.out.println( "Refresh Tree at Element: " + ((INavigatorElement) m_navElement).getName() );
    PlatformUI.getWorkbench().getDisplay().syncExec( new Runnable()
    {

      @Override
      public void run()
      {
        m_viewer.refresh( m_navElement, false );
      }

    } );

  }

  /*
   * @see IDeltaListener#remove(DeltaEvent)
   */
  public void remove( DeltaEvent event )
  {
    if ( _debug )
      System.out.println( "Remove from Tree!" );
    add( event );
  }

  /*
   * @see IDeltaListener#update(DeltaEvent)
   */
  public void update( DeltaEvent event )
  {
    m_navElement = event.receiver();
    if ( _debug )
      System.out.println( "Update Tree at Element: " + ((INavigatorElement) m_navElement).getName() );
    Display display = PlatformUI.getWorkbench().getDisplay();
    if ( !display.isDisposed() )
    {
      display.syncExec( new Runnable()
      {

        @Override
        public void run()
        {
          m_viewer.update( m_navElement, null );
        }

      } );
    }

  }

}
