/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * Contact Information:
 * Dambach-Werke GmbH
 * Adolf-Dambach-Strae
 * 76571 Gaggenau
 * Phone: +49-7225-64-01
 * Fax: +49-7225-64-300
 * mailto: info@els.dambach.de
 */

package de.bsvrz.sba.plugin.parameter.steuerungskern.parameter.entfernungsangaben;

import java.util.Vector;

import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.Data.Array;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.sba.plugin.parameter.steuerungskern.daten.WVZInhalt;

/**
 * Klasse realisiert eine Treppenfunktion fr die Entfernungsangaben
 * @version $Revision: 1.1 $ / $Date: 2013/06/05 14:17:02 $ / ($Author: ObertM $)
 * @author Dambach-Werke GmbH
 * @author Matthias Obert
 */
public class TreppenFunktion
{
  /**
   * Bezeichnung fr die Treppenfunktion.
   */
  private String m_bezeichnung;
  
  /**
   * Vektor mit Stufen fr die Entfernungsangaben die die Treppenfunktion realisieren.
   */
  private Vector<Stufe> m_stufen = new Vector<Stufe>();

  /**
   * Konstruktor der Klasse
   * @param bezeichnung Bezeichnung der Treppenfunktion
   */
  public TreppenFunktion (String bezeichnung)
  {
    m_bezeichnung = bezeichnung;
  }

  /**
   * Parameterloser Konstruktor der Klasse
   */
  public TreppenFunktion()
  {
  }

  /**
   * Methode liefert die Variable m_bezeichnung zurck
   * @return Variable m_bezeichnung
   */
  public String getBezeichnung()
  {
    return m_bezeichnung;
  }

  /**
   * Methode setzt die Variable m_bezeichnung
   * @param bezeichnung Wert auf den die Variable m_bezeichnung gesetzt werden soll
   */
  public void setBezeichnung( String bezeichnung )
  {
    m_bezeichnung = bezeichnung;
  }

  /**
   * Methode liefert die Variable m_stufen zurck
   * @return Variable m_stufen
   */
  public Vector<Stufe> getStufen()
  {
    return m_stufen;
  }

  /**
   * Methode setzt die Variable m_stufen
   * @param stufen Wert auf den die Variable m_stufen gesetzt werden soll
   */
  public void setStufen( Vector<Stufe> stufen )
  {
    m_stufen = stufen;
  }

  /**
   * Methode zum Auswerten einer Attributliste 'atl.sswSbaEntferungsAngabeTreppenFunktion'
   * @param data Empfangene Daten vom DaV
   */
  public void auswerteAttributgruppe( Data data )
  {
    synchronized (m_stufen)
    {
      // Bezeichnung
      
      m_bezeichnung = data.getTextValue( "Bezeichnung" ).getText();
      
      // Stufen
      
      Array a = data.getArray( "Stufen" );
      int anzahl = a.getLength();
      
      m_stufen.clear();
      for (int x = 0; x < anzahl; ++x)
      {
        Stufe stufe = new Stufe ();
        m_stufen.add( stufe );
        
        stufe.auswerteAttributgruppe (a.getItem( x ));
      }
    }
  }
  
  /**
   * Methode zum Ausgeben der Klasse als String
   */
  @Override
  public String toString ()
  {
    String s = "";
    
    s += "Bezeichnung: " + getBezeichnung() + "\n";
    s += "Stufen:\n";
    
    int x = 1;
    for (Stufe st : getStufen())
    {
      s += "Stufe " + x + ": " + st.toString();
      x++;
    }
    
    return s;
  }

  /**
   * Methode liefert den passenden WVZ-Inhalt der in dieser Treppenfunktion zu einer bestimmten
   * Entferung gehrt. Wird kein geeigneter WVZ-Inhalt gefunden wird null zurckgeliefert.
   * @param entfernung zu prfende Entfernung
   * @return passender WVZ-Inhalt oder null
   */
  public WVZInhalt getDarstellungsObjekt( long entfernung )
  {
    synchronized (m_stufen)
    {
      for (Stufe st : m_stufen)
      {
        if ((st.getVon() <= entfernung) && (st.getBis() >= entfernung))
          return st.getWvzInhalt();
      }
    }
    
    return null;
  }

  /**
   * Methode liefert die Stufe die zu einem WVZ-Inhalt gehrt  
   * @param wvzInhalt WVZ-Inhalt dessen Stufe gesucht wird
   * @return Stufe sofern vorhanden, sonst null
   */
  public Stufe getStufe( SystemObject wvzInhalt )
  {
    synchronized (m_stufen)
    {
      for (Stufe st : m_stufen)
      {
        if (st.getWvzInhalt() == wvzInhalt )
          return st;
      }
    } 
    
    return null;
  }

  /**
   * Methode fgt eine Stufe der Treppenfunktion hinzu
   * @param von unterer Schwellwert der Stufe
   * @param bis oberer Schwellwert der Stufe
   * @param wvzInhalt dazugehrender WVZ-Inhalt
   */
  public void addStufe( long von, long bis, WVZInhalt wvzInhalt )
  {
    Stufe stufe = new Stufe ();
    
    stufe.setVon( von );
    stufe.setBis( bis );
    stufe.setWvzInhalt( wvzInhalt );

    synchronized (m_stufen)
    {
      m_stufen.add( stufe );
    }
  }
}
