/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * Contact Information:
 * Dambach-Werke GmbH
 * Adolf-Dambach-Strae
 * 76571 Gaggenau
 * Phone: +49-7225-64-01
 * Fax: +49-7225-64-300
 * mailto: info@els.dambach.de
 */

package de.bsvrz.sba.plugin.parameter.steuerungskern.parameter.laengsquerabgleich.trichterung;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Vector;
import java.util.Map.Entry;

import de.bsvrz.buv.rw.rw.dav.DaVVerbindung;
import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.ReceiverRole;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.Data.Array;
import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.sba.plugin.parameter.steuerungskern.daten.WVZInhalt;
import de.bsvrz.sba.plugin.parameter.steuerungskern.daten.Richtung.ArbeitsRichtungVN;
import de.bsvrz.sba.plugin.parameter.steuerungskern.dav.DaVKommunikationSBA;
import de.bsvrz.sba.plugin.parameter.steuerungskern.dav.SendeParameter;
import de.bsvrz.sba.plugin.parameter.steuerungskern.konfig.Konfig;
import de.bsvrz.sba.plugin.parameter.steuerungskern.tools.Tools;

/**
 * Klasse enthlt die Parameter fr die Regelklasse "Lngsabgleichs- und Querabgleichsregeln", 
 * Regel "Trichterung".<br>
 * Regel:<br>
 * Wenn Anzeigeinhalt = "X" und <br>
 * Inhalt des Vorgnger/Nachfolger in Lngsrichtung = "Y"<br> 
 * dann fr diesen Vorgnger/Nachfolger in Lngsrichtung "Z" anfordern.<br>
 * @author Swarco Traffic System GmbH
 * @author Matthias Obert
 *
 */
public class VerwaltungParaTrichterung implements Runnable
{
  /**
   * Debug Ausgabe
   */
  private boolean _debug = false;
  
  /**
   * Allgemeiner Schaltgrund fr die Regelklasse
   */
  private Aspect m_schaltgrundAllgemein;
  
  /**
   * Parametersatz fr die Anlage
   */
  private RegelnProAnzeigeTyp m_parameterAnlage = new RegelnProAnzeigeTyp ();
  
  /**
   * Parametersatz pro Anzeigequerschnitt
   */
  private HashMap<SystemObject, RegelnProAnzeigeTyp> m_parameterProAnzeigeQuerschnitt = new HashMap<SystemObject, RegelnProAnzeigeTyp>();
  
  /**
   * Parametersatz pro Anzeige
   */
  private HashMap<SystemObject, RegelnProAnzeigeTyp> m_parameterProAnzeige = new HashMap<SystemObject, RegelnProAnzeigeTyp>();
  
  /**
   * Listener
   */
  private Vector<PropertyChangeListener> _angemeldeteListener = new Vector<PropertyChangeListener>();

  /**
   * Objekt zum Anstossen des Threads
   */
  private Object _merker = new Object ();

  /**
   * Mutex fr den synchronisierten Zugriff auf die Datenstruktur
   */
  private Object m_mutex = new Object();

  /**
   * Pid der Anlage
   */
  private String m_anlage;

  /**
   * Pid der Attributgruppe
   */
  private String _pidAtg = "atg.sswSbaTrichterung";
  
  /**
   * Pid des Defaultaspekts
   */
  public static String _pidDefaultAspekt = "asp.sswSbaTrichterung";
  
  /**
   * Pid der SBA
   */
  private String _pidSBA;
  
  /**
   * Einzige Instanz der Klasse
   */
  private static VerwaltungParaTrichterung _instance = null;

  /**
   * Privater Konstruktor der Klasse
   */
  private VerwaltungParaTrichterung ()
  {
  	_pidSBA = Konfig.getInstance().getPidSBA();
    
    m_schaltgrundAllgemein = Konfig.getInstance().getAspekt(_pidDefaultAspekt);
    
    /*
     * Datrn lesen ber DaVKommunikation
     */
  	final DaVKommunikationSBA kom = DaVKommunikationSBA.getInstanz();
  	kom.addDatenArt(_pidSBA, _pidAtg, "asp.parameterSoll", ReceiverRole.receiver());
  	
  	kom.addPropertyChangeListener(new PropertyChangeListener()
		{
			@Override
			public void propertyChange(PropertyChangeEvent evt)
			{
				if (evt.getPropertyName().equals(_pidAtg))
				{
					ResultData resultData = kom.getData(_pidAtg);
					if (resultData != null)
						auswerteAttributgruppe(resultData);
				}
			}
		});

		ResultData resultData = kom.getData(_pidAtg);
		if (resultData != null)
			auswerteAttributgruppe(resultData);

    // Thread zum Benachrichtigen der Listener
    
    Thread t = new Thread (this);
    t.setName(this.getClass().getName());
    t.start();
  }

  /**
   * Methode liefert die Singleton Instanz der Klasse
   * @return Singleton Instanz der Klasse
   */
  public static VerwaltungParaTrichterung getInstance ()
  {
    if (_instance == null)
    {
      synchronized (VerwaltungParaTrichterung.class)
      {
        if (_instance == null)
        {
          _instance = new VerwaltungParaTrichterung ();
        }
      }
    }
    
    return _instance;
  }

  /**
   * Methode liefert den Schaltgrund fr die Regelklasse
   * @return Schaltgrund fr die Regelklasse
   */
  public Aspect getSchaltgrundAllgemein()
  {
    return m_schaltgrundAllgemein;
  }

  /**
   * Methode setzt des Schaltgrund fr dei Regelklasse
   * @param schaltgrund Schaltgrund fr dei Regelklasse
   */
  public void setSchaltgrundAllgemein( Aspect schaltgrund )
  {
    m_schaltgrundAllgemein = schaltgrund;
  }
  
  /**
   * Methode zum Auswerten der Attributgruppe 'atg.sswSbaTrichterung'
   * @param dat empfangender Datensatz
   */
  public void auswerteAttributgruppe( ResultData dat )
  {
    synchronized (m_mutex)
    {
      if (!dat.hasData())
      {
        m_schaltgrundAllgemein = null;
        m_parameterAnlage.clear();
        m_parameterProAnzeigeQuerschnitt.clear();
        m_parameterProAnzeige.clear();
        
      }
      else
      {
        Data data = dat.getData();
          
        m_schaltgrundAllgemein = (Aspect) data.getReferenceValue( "SchaltgrundAllgemein" ).getSystemObject();
        
        Array a = data.getArray( "AnlageRegeln" );
        auswerteAttributgruppeAnlage( a );
        
        a = data.getArray( "AnzeigeQuerschnitt" );
        auswerteAttributgruppeQuerschnitt (a);
        
        a = data.getArray( "Anzeige");
        auswerteAttributgruppeAnzeige (a);
      }
    }
    
    m_schaltgrundAllgemein = Konfig.getInstance().getAspekt(_pidDefaultAspekt);
    
    benachrichtigeListener();
  }

  /**
   * Methode zum Auswerten der fr die gesamte Anlage geltenden Parameter
   * @param array Array mit Parametern
   */
  private void auswerteAttributgruppeAnlage (Array array)
  {
    m_parameterAnlage.auswerteAttributgruppe( array, m_schaltgrundAllgemein );
  }

  /**
   * Methode zum Auswerten der Anzeigequerschnitt spezifischen Parameter
   * @param array Array mit Parametern fr die Anzeigequerschnitte
   */
  private void auswerteAttributgruppeQuerschnitt( Array array )
  {
    int anzahl = array.getLength(); 
    
    synchronized (m_parameterProAnzeigeQuerschnitt)
    {
      m_parameterProAnzeigeQuerschnitt.clear();
      for (int x = 0; x < anzahl; ++x)
      {
        Data d1 = array.getItem( x );
        
        SystemObject anzeigeQuerschnitt = d1.getReferenceValue( "AnzeigeQuerschnitt" ).getSystemObject();
        Array a = d1.getArray( "AnzeigeQuerschnittRegeln" );
        
        RegelnProAnzeigeTyp regeln  = new RegelnProAnzeigeTyp ();
        regeln.auswerteAttributgruppe( a, m_schaltgrundAllgemein );
        
        m_parameterProAnzeigeQuerschnitt.put (anzeigeQuerschnitt, regeln);
      }
    }
  }

  /**
   * Methode zum Auswerten der Anzeige spezifischen Parameter
   * @param array Array mit Parametern fr die Anzeige
   */
  private void auswerteAttributgruppeAnzeige (Array array )
  {
    int anzahl = array.getLength(); 
    
    synchronized (m_parameterProAnzeige)
    {
      m_parameterProAnzeige.clear();
      for (int x = 0; x < anzahl; ++x)
      {
        Data d1 = array.getItem( x );
        
        SystemObject anzeige = d1.getReferenceValue( "Anzeige" ).getSystemObject();
        Array a = d1.getArray( "AnzeigeRegeln" );
        
        RegelnProAnzeigeTyp regeln  = new RegelnProAnzeigeTyp ();
        regeln.auswerteAttributgruppe( a,m_schaltgrundAllgemein  );
        
        m_parameterProAnzeige.put (anzeige, regeln);
      }
    }
  }

  /**
   * Methode fgt eine Regel fr die allgemeinen Parameter der Anlage hinzu
   * @param wvzTyp WVZ Typ fr den die Regel gelten soll
   * @param x Anzeigeinhalt der aktuell betrachteten Anzeige 
   * @param y Anzeigeinhalt des Vorgnger/Nachfolgers
   * @param z Angeforderter Anzeigeinhalt
   * @param richtung Richtung in der die Regel arbeitet
   * @param schaltgrund Schaltgrund unter dem diese Regel publiziert wird
   */
  public void addRegelAllgemein( SystemObject wvzTyp, WVZInhalt x, WVZInhalt y, WVZInhalt z, ArbeitsRichtungVN richtung, Aspect schaltgrund )
  {
    ListeMitRegeln regeln = m_parameterAnlage.getParameter( wvzTyp );
    regeln.addRegel( x, y, z, richtung, schaltgrund );
  }
  
  /**
   * Methode liefert den Inhalt der Klasse als String
   */
  public String toString ()
  {
    String s = "";
    
    s += "Parameter Trichterung:\n";
    s += "======================\n";  
    
    s += "Parameter Anlage:\n";
    s += "=================\n";
    s += m_parameterAnlage.toString();
    
    s += "Parameter pro Anzeigequerschnitt:\n";
    s += "=================================\n";
    Iterator<Entry<SystemObject, RegelnProAnzeigeTyp>> it = m_parameterProAnzeigeQuerschnitt.entrySet().iterator();
    while (it.hasNext())
    {
      Map.Entry<SystemObject, RegelnProAnzeigeTyp> entry = it.next();
      
      SystemObject anzeigeQuerschnitt = (SystemObject) entry.getKey();
      RegelnProAnzeigeTyp regeln = (RegelnProAnzeigeTyp) entry.getValue();
      
      s += Tools.getObjektName( anzeigeQuerschnitt )  + "\n";
      s += regeln.toString();
    }

    s += "Parameter pro Anzeige:\n";
    s += "======================\n";
    it = m_parameterProAnzeige.entrySet().iterator();
    while (it.hasNext())
    {
      Map.Entry<SystemObject, RegelnProAnzeigeTyp> entry = it.next();
      
      SystemObject anzeige = (SystemObject) entry.getKey();
      RegelnProAnzeigeTyp regeln = (RegelnProAnzeigeTyp) entry.getValue();
      
      s += Tools.getObjektName( anzeige )  + "\n";
      s += regeln.toString();
    }

    return s;
  }
  
  /**
   * Methode zum Erzeugen einer Attributgruppe mit den Parametern
   * @return Attributgruppe mit den Parametern
   */
  public Data erzeugeAttributgruppe ()
  {
	  ClientDavInterface connection = DaVVerbindung.getInstanz().getDaVVerbindung();
	
    if (connection == null)
      return null;

    bereinigeParameter ();
    
    String pidAtg = _pidAtg ;
    
    // Attributgruppe zusammensetzen und versenden
      
    AttributeGroup atg = (AttributeGroup) connection.getDataModel().getAttributeGroup(pidAtg);
    Data data = connection.createData(atg );

    data.getReferenceValue( "SchaltgrundAllgemein" ).setSystemObject( getSchaltgrundAllgemein() );

    Array anlageRegeln = data.getArray( "AnlageRegeln" );
    erzeugeAnlageRegeln (anlageRegeln);

    Array anzeigeQuerschnitt = data.getArray( "AnzeigeQuerschnitt" );
    erzeugeAnzeigeQuerschnittRegeln (anzeigeQuerschnitt);
    
    Array anzeige = data.getArray( "Anzeige" );
    erzeugeAnzeigeRegeln (anzeige);
    
    return data;
  }

  /**
   * Methode lscht leere Parameterfelder. Dies ist notwendig, da sonst die Steuerung Problem
   * mit der Bestimmung der richtigen Parameterstze hat
   */
  private void bereinigeParameter()
  {
  	bereinigeRegelProAnzeigeTyp(getParameterProAnzeigeQuerschnitt());

  	bereinigeRegelProAnzeigeTyp(getParameterProAnzeige());
  }
  
  /**
   * Methode bereinigt die Parameter. Dabei werden Anzeigetypen fr die keine Regeln definiert sind
   * aus dem Parametersatz entfern.
   * @param regelnProAnzeigeTyp zu bereinigende Regeln
   */
  private void bereinigeRegelProAnzeigeTyp (HashMap<SystemObject, RegelnProAnzeigeTyp> regelnProAnzeigeTyp)
  {
  	List<SystemObject> zuLoeschen = new ArrayList<SystemObject>();
  	
	  Iterator<Entry<SystemObject, RegelnProAnzeigeTyp>> it = regelnProAnzeigeTyp.entrySet().iterator();
	  while (it.hasNext())
	  {
	  	Entry<SystemObject,RegelnProAnzeigeTyp> entry = it.next();
	  	
	  	SystemObject anzeigeQuerschnitt = entry.getKey();
	  	RegelnProAnzeigeTyp regeln = entry.getValue();
	  	
	  	regeln.bereinigeParameter();
	  	if (regeln.getParameter().size() == 0)
	  		zuLoeschen.add (anzeigeQuerschnitt);
	  }

	  for (SystemObject so : zuLoeschen)
	  	regelnProAnzeigeTyp.remove(so);
  }

	/**
   * Methode schreibt die Anlageregeln in ein Array einer Attributgruppe
   * @param array Array fr die Regeln
   */
  private void erzeugeAnlageRegeln( Array array )
  {
    RegelnProAnzeigeTyp rpat = getParameterAnlage();
    erzeugeRegeln( array, rpat );
  }

  /**
   * Methode schreibt die Anzeigequerschnittregeln in ein Array einer Attributgruppe
   * @param array Array fr die Regeln
   */
  private void erzeugeAnzeigeQuerschnittRegeln( Array array )
  {
    HashMap<SystemObject, RegelnProAnzeigeTyp> paraAq = getParameterProAnzeigeQuerschnitt();
    int anzahlAQ = paraAq.size();
    
    array.setLength( anzahlAQ );
    
    int i=0;
    Iterator<Entry<SystemObject, RegelnProAnzeigeTyp>> it = paraAq.entrySet().iterator();
    while (it.hasNext())
    {
      Entry<SystemObject, RegelnProAnzeigeTyp> entry = it.next();
      
      SystemObject anzeigeQuerschnitt = entry.getKey();
      RegelnProAnzeigeTyp rpat = entry.getValue();
      
      Data dataAQ = array.getItem( i++ );
      
      dataAQ.getReferenceValue( "AnzeigeQuerschnitt" ).setSystemObject( anzeigeQuerschnitt );
      Array arrayRegeln = dataAQ.getArray( "AnzeigeQuerschnittRegeln" );
      
      erzeugeRegeln (arrayRegeln, rpat);
    }
  }

  /**
   * Methode schreibt die Anzeigeregeln in ein Array einer Attributgruppe
   * @param array Array fr die Regeln
   */
  private void erzeugeAnzeigeRegeln( Array array )
  {
    HashMap<SystemObject, RegelnProAnzeigeTyp> paraAnzeige = getParameterProAnzeige();
    
    int anzahlAnzeigen = paraAnzeige.size();

    array.setLength( anzahlAnzeigen );
    
    int i=0;
    Iterator<Entry<SystemObject, RegelnProAnzeigeTyp>> it = paraAnzeige.entrySet().iterator();
    while (it.hasNext())
    {
      Entry<SystemObject, RegelnProAnzeigeTyp> entry = it.next();
      
      SystemObject anzeige = entry.getKey();
      RegelnProAnzeigeTyp rpat = entry.getValue();

      Data dataAQ = array.getItem( i++ );
      
      dataAQ.getReferenceValue( "Anzeige" ).setSystemObject( anzeige );
      Array arrayRegeln = dataAQ.getArray( "AnzeigeRegeln" );

      erzeugeRegeln (arrayRegeln, rpat);
    }
  }

  /**
   * Methode schreibt die Regeln pro Anzeigetyp in ein Array
   * @param array Array in das die Regeln geschrieben werden solle
   * @param rpat Regeln
   */
  private void erzeugeRegeln( Array array, RegelnProAnzeigeTyp rpat )
  {
    HashMap<SystemObject, ListeMitRegeln> p = rpat.getParameter();
    int anzahl = p.size();
    
    array.setLength( anzahl );
    
    int i=0;
    Iterator<Entry<SystemObject, ListeMitRegeln>> it = p.entrySet().iterator();
    while (it.hasNext())
    {
      Entry<SystemObject, ListeMitRegeln> entry = it.next();
      
      SystemObject anzeigeTyp = entry.getKey();
      ListeMitRegeln regeln = entry.getValue();
      
      Data aktuell = array.getItem( i++ );
      
      aktuell.getReferenceValue( "AnzeigeTyp" ).setSystemObject( anzeigeTyp );
      Array r = aktuell.getArray( "Regeln" );
      
      Vector<RegelTrichterung> parameter = regeln.getParameter();
      r.setLength( parameter.size() );
      
      for (int j=0; j<parameter.size(); ++j)
      {
        Data d = r.getItem( j );
        RegelTrichterung rv = parameter.get( j );
        
        d.getReferenceValue( "SchaltgrundRegel" ).setSystemObject( rv.getSchaltgrundRegel() );
        d.getUnscaledValue ( "VorgngerNachfolger" ).set (rv.getRichtung().getKodierung());
        
        WVZInhalt x = rv.getAnzeigenInhalt();
        x.erzeugeAttributgruppe( d.getItem( "Anzeigeinhalt" ) );
        
        WVZInhalt y = rv.getAnzeigenInhaltVorgaengerNachfolger();
        y.erzeugeAttributgruppe( d.getItem( "AnzeigeinhaltVorgngerNachfolger" ) );
        
        WVZInhalt z = rv.getAngeforderterAnzeigenInhalt();
        z.erzeugeAttributgruppe( d.getItem( "AngeforderterAnzeigeInhalt" ) );
      }
    }
  }

  /**
   * Methode liefert die Variable m_parameterAnlage zurck
   * @return Variable m_parameterAnlage
   */
  public RegelnProAnzeigeTyp getParameterAnlage()
  {
    return m_parameterAnlage;
  }

  /**
   * Methode liefert die Variable m_parameterProAnzeigeQuerschnitt zurck
   * @return Variable m_parameterProAnzeigeQuerschnitt
   */
  public HashMap<SystemObject, RegelnProAnzeigeTyp> getParameterProAnzeigeQuerschnitt()
  {
    return m_parameterProAnzeigeQuerschnitt;
  }

  /**
   * Methode liefert die Variable m_parameterProAnzeige zurck
   * @return Variable m_parameterProAnzeige
   */
  public HashMap<SystemObject, RegelnProAnzeigeTyp> getParameterProAnzeige()
  {
    return m_parameterProAnzeige;
  }

  /**
   * Methode liefert die Parameter als Attributgruppe fr den DaV
   * @return Attributgruppe
   */
  public Data getData ()
  {
  	return erzeugeAttributgruppe();
  }
  
  /**
   * Methode zum Speichern der Parameter im DaV
   */
  public void speichereParameterInDaV ()
  {
    SendeParameter sende = new SendeParameter (_pidAtg);
    sende.sendeParameter(getData());
  }
  
  @Override
  public void run() 
  {
    while (true)
    {
      // Warten aus nderung des aktuell anstehenden Programmes
      
      try
      {
        synchronized (_merker)
        {
          _merker.wait();
        }
      } 
      catch (InterruptedException e)
      {
        e.printStackTrace();
      }

      firePropertyChange ();
    }
  }

  /**
   * Methode benachrichtigt die angemedeten Listener
   */
  private void firePropertyChange ()
  {
    PropertyChangeEvent event = new PropertyChangeEvent (this.getClass(), "ParameterTrichterung", null, null);

    if (event != null)
    {
      synchronized (_angemeldeteListener)
      {
        Iterator<PropertyChangeListener> it = _angemeldeteListener.iterator();
        while (it.hasNext())
        {
          PropertyChangeListener l = it.next();
          l.propertyChange(event);
        }
      }
    }
  }

  /**
   * Methode fgt einen Listener der Klasse hinzu
   * @param listener Listern der Klasse PropertyChangeListener
   */
  public void addPropertyChangeListener (PropertyChangeListener listener)
  {
    synchronized (_angemeldeteListener)
    {
      _angemeldeteListener.add(listener);
    }

    benachrichtigeListener();
  }
  
  /**
   * Methode entfernt einen Listener aus der Klasse
   * @param listener Listern der Klasse PropertyChangeListener
   */
  public void removePropertyChangeListener (PropertyChangeListener listener)
  {
    synchronized (_angemeldeteListener)
    {
      if (_angemeldeteListener.contains(listener))
        _angemeldeteListener.remove(listener);
    }
  }

  private void benachrichtigeListener ()
  {
    synchronized (_merker)
    {
      _merker.notify();
    }
  }

  /**
   * Methode liefert die Parameter zu einer bestimmten Pid
   * @param pid Pid des gesuchten Objekts, dies kann die Pid der SBA, die Pid eines Anzeigequerschnitts oder
   * die Pid einer Anzeige sein.
   * @return Parameter zu der Pid
   */
  public RegelnProAnzeigeTyp getParameter (String pid)
  {
    /*
     * Parameter Anlage
     */
    if (pid.equals(_pidSBA))
      return getParameterAnlage();
    
    /*
     * Parameter Anzeigequerschnitt
     */
    for (SystemObject so : m_parameterProAnzeigeQuerschnitt.keySet())
    {
      if (pid.equals(so.getPid()))
        return m_parameterProAnzeigeQuerschnitt.get(so);
    }
    
    /*
     * Parameter Anzeige
     */
    for (SystemObject so : m_parameterProAnzeige.keySet())
    {
      if (pid.equals(so.getPid()))
        return m_parameterProAnzeige.get(so);
    }
    
    /*
     * Parametersatz noch nicht definiert
     */
   
    return new RegelnProAnzeigeTyp();
  }

  /**
   * Methode bernimmt die Parameter fr eine bestimmte Pid
   * @param pid Pid
   * @param regeln Regeln
   */
	public void setParameter(String pid, RegelnProAnzeigeTyp regeln)
  {
    ClientDavInterface connection = DaVVerbindung.getInstanz().getDaVVerbindung();

    /*
     * Parameter Anlage
     */
    if (pid.equals(_pidSBA))
    {
      m_parameterAnlage = regeln;
    }
    else if (connection != null)
    {
    	/*
    	 * Prfen ob Anzeigequerschnitt oder Anzeige
    	 */
    	
    	SystemObject so = connection.getDataModel().getObject(pid);
    	if (so != null)
    	{
    		if (so.getType().getPid().equals("typ.anzeigeQuerschnitt"))
    			m_parameterProAnzeigeQuerschnitt.put(so, regeln);

    		if (so.getType().getPid().equals("typ.anzeige"))
    			m_parameterProAnzeige.put(so, regeln);
    	}
    }
    
    benachrichtigeListener();
  }
}
