package de.bsvrz.sba.plugin.parameter.steuerungskern.parameter.prio;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.Map.Entry;

import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.Data.Array;
import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.sba.plugin.parameter.steuerungskern.daten.WVZInhalt;

/**
 * Klasse enthlt eine Regel fr die Priorisierung. Eine Regel besteht dabei aus einer 
 * 'Defaultregel' und u.U. unterschiedlichen Prioritten fr unterschiedliche Aspekte 
 * (Schaltgrnde).
 * @author   Dambach Werke GmbH
 */
class RegelPriorisierung
{
	/**
	 * WVZ Inhalt fr den die Regel gelten soll
	 */
	private WVZInhalt _wvzInhalt;
	
  /**
   * Prioritt
   */
  private int m_prioritaet = -1;
  
  /**
   * Liste mit zugeordnete Prioritten unter unterschiedlichen Aspekten
   */
  private HashMap<Aspect, Integer> m_prioritaetAspekte = new HashMap<Aspect, Integer>();

  /**
   * Methode liefert die Prioritt fr einen WVZ-Inhalt
   * @param aspekt Aspekt fr den Prioritt gesucht wird
   * @return Prioritt, wenn keine definiert ist -1
   */
  public int getPrio (Aspect aspekt)
  {
    int prio = -1;
    
    synchronized (m_prioritaetAspekte)
    {
      if (m_prioritaetAspekte.containsKey( aspekt ))
        prio = m_prioritaetAspekte.get( aspekt );
      else
        prio = m_prioritaet;
    }
    
    return prio;
  }
  
  /**
   * Methode zum Auswerten der Attributgruppe
   * @param data vom DaV empfangene Daten
   */
  public void auswerteAttributGruppe (Data data)
  {
    m_prioritaet = data.getItem( "Prioritt" ).asUnscaledValue().intValue();

    synchronized (m_prioritaetAspekte)
    {
      m_prioritaetAspekte.clear();
      Array prioritaeten = data.getArray( "Schaltgrnde" );

      int anzahl = prioritaeten.getLength();
      for (int x = 0; x < anzahl; ++x)
      {
        Data d = prioritaeten.getItem( x );
        
        SystemObject schaltgrund = d.getReferenceValue( "Schaltgrund" ).getSystemObject();
        Aspect aspekt = (Aspect) schaltgrund;
        
        int prioritaet = d.getItem( "Prioritt" ).asUnscaledValue().intValue();
        
        m_prioritaetAspekte.put ( aspekt, prioritaet );
      }
    }
  }
  
  /**
   * Methode zum Ausgeben der Klasse als String
   * @return Klasse als String
   */
  public String toString ()
  {
    String s = "";
    
    s += "RegelPriorisierung:\n";
    s += "Prioritt: " + m_prioritaet + "\n";
    s += "Prioritt pro Aspekt:\n";
    
    synchronized (m_prioritaetAspekte)
    {
      Iterator<Entry<Aspect, Integer>> it = m_prioritaetAspekte.entrySet().iterator();
      while (it.hasNext())
      {
        Map.Entry<Aspect, Integer> entry = it.next();
        Aspect a = (Aspect) entry.getKey();
        Integer prio = (Integer) entry.getValue();
        
        if (a == null)
          s += "-" + "null" + ": " + prio + "\n";
        else
          s += "-" + a.getPid() + ": " + prio + "\n";
      }
    }
    
    return s;
  }

  /**
   * Methode liefert die Variable m_prioritaet zurck
   * @return Variable m_prioritaet
   */
  public int getPrioritaet()
  {
    return m_prioritaet;
  }

  /**
   * Methode setzt die Variable m_prioritaet
   * @param prioritaet Wert auf den die Variable m_prioritaet gesetzt werden soll
   */
  public void setPrioritaet( int prioritaet )
  {
    m_prioritaet = prioritaet;
  }

  /**
   * Methode liefert die Variable m_prioritaetAspekte zurck
   * @return Variable m_prioritaetAspekte
   */
  public HashMap<Aspect, Integer> getPrioritaetAspekte()
  {
  	synchronized (m_prioritaetAspekte)
    {
      return m_prioritaetAspekte;
    }
  }

  /**
   * Methode setzt die Variable m_prioritaetAspekte
   * @param prioritaetAspekte Wert auf den die Variable m_prioritaetAspekte gesetzt werden soll
   */
  public void setPrioritaetAspekte( HashMap<Aspect, Integer> prioritaetAspekte )
  {
  	synchronized (m_prioritaetAspekte)
    {
	    m_prioritaetAspekte.clear();
      Iterator<Entry<Aspect, Integer>> it = m_prioritaetAspekte.entrySet().iterator();
      while (it.hasNext())
      {
        Entry<Aspect, Integer> entry = it.next();
        m_prioritaetAspekte.put(entry.getKey(), entry.getValue());
      }
    }
  	
    m_prioritaetAspekte = prioritaetAspekte;
  }

  /**
   * Methode fgt einen Schaltgrund mit einer Prioritt hinzu
   * @param schaltgrund Schaltgrund
   * @param prio Prioritt
   */
	public void addAspekt(Aspect schaltgrund, int prio)
  {
		synchronized (m_prioritaetAspekte)
    {
			m_prioritaetAspekte.put(schaltgrund, prio);
    }
  }

	/**
   * Methode liefert wvzInhalt zurck
   * @return wvzInhalt
   */
  public WVZInhalt getWvzInhalt()
  {
  	return _wvzInhalt;
  }

	/**
   * Methode setzt die Variable wvzInhalt
   * @param wvzInhalt zu bernehmender Wert fr wvzInhalt
   */
  public void setWvzInhalt(WVZInhalt wvzInhalt)
  {
  	_wvzInhalt = wvzInhalt;
  }
}
